
import java.awt.Color;
import java.awt.Paint;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.BasicStroke;
import java.awt.image.BufferedImage;
import java.awt.image.AffineTransformOp;
import java.awt.geom.AffineTransform;

import java.util.Observable;
import java.util.Random;

// The painting itself, which contains a buffered bitmap
// image
public class GraphicsModel extends Observable {

	private BufferedImage image;
	// a second buffer used for zooming
	private BufferedImage image2;

	private Graphics2D graphics;

	private final Color background = Color.white;
    private final Color foreground = Color.black;

    final static BasicStroke wideStroke = new BasicStroke(8.0f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);   
	
	public GraphicsModel(int width, int height) {
		image = new BufferedImage(width, height, BufferedImage.TYPE_INT_BGR);
		image2 = new BufferedImage(width, height, BufferedImage.TYPE_INT_BGR);
		initPainting();
	}
	
	public void initPainting() {
		graphics = image.createGraphics();
		graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
	    graphics.setPaint(foreground);
	    graphics.setBackground(background);
	    graphics.setStroke(wideStroke);
	    graphics.clearRect(0,0,getWidth(),getHeight());
	}
	
	public void paintDot(int x, int y) {
		int size = 10;
		graphics.fillOval(x-size/2,y-size/2,size,size);
		setChanged();
		notifyObservers();
	}
	
	public void paintRandomColoredDot(int x, int y) {
		Random generator = new Random();
		graphics.setPaint(new Color(generator.nextInt()));
		paintDot(x, y);
		graphics.setPaint(foreground);
	}
	
	public void paintImage(BufferedImage im, int x, int y) {
		graphics.drawImage(im, null, x, y);
		setChanged();
		notifyObservers();
	}
	
	public int getHeight() {
		return image.getHeight();
	}
	
	public int getWidth() {
		return image.getWidth();
	}
	
	public void paintLine(int x1, int y1, int x2, int y2) {
		graphics.drawLine(x1, y1, x2, y2);
		setChanged();
		notifyObservers();
	}
	
	public void zoom(double factor, int x_center, int y_center) {
		scale(
			(int)(x_center * (1-factor)),
			(int)(y_center * (1-factor)),
			factor, factor);
		
	}
	
	private void scale(int x, int y, double xamount, double yamount) {
		image.copyData(image2.getRaster());
		graphics.drawImage(image2, new AffineTransformOp(AffineTransform.getScaleInstance(xamount, yamount), null), x, y);
		setChanged();
		notifyObservers();
	}
	
	public BufferedImage getImage() {
		return image;
	}

	public Color getForeground(){
		return this.foreground;
	}
	
	public Color getBackGround() {
		return this.background;
	}
	
}
