import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JComponent;

// Represents the canvas which draws
// its own buffered graphicsmodel, to which
// it registers itself as an observer
public class PaintCanvas extends JComponent implements Observer {

    private int height;
    private int width;

    private GraphicsModel graphicsModel;
    private StrokeListener listener;
    
    private static final int DEFAULT_HEIGHT = 500;
    private static final int DEFAULT_WIDTH = 500;
	
    public PaintCanvas(int width, int height) {
    	super();
    	this.height = height;
    	this.width = width;
    	initialize();
    }

    public PaintCanvas() {
		this(DEFAULT_WIDTH, DEFAULT_HEIGHT);
	}

    public void update(Observable model, Object arg) {
    	// repaints itself when buffered image has changed
		repaint(new Rectangle(width, height));
	}
    
    private void initialize() {
		setPreferredSize(new Dimension(width,height));
		setMaximumSize(new Dimension(width, height));
		setMinimumSize(new Dimension(width, height));
		setOpaque(true);
		setVisible(true);

		graphicsModel = new GraphicsModel(width,height);
    	graphicsModel.addObserver(this);

    	listener = new StrokeListener(graphicsModel);
    	addMouseListener(listener);
    	addMouseMotionListener(listener);
    	addMouseWheelListener(listener);
    }
    
	protected void paintComponent(Graphics g) {
		Graphics2D graphics = (Graphics2D) g;
		// just draws the image
	    graphics.drawImage(graphicsModel.getImage(), null, 0, 0);
	}
}
